package ch.rakudave.jnetmap.view.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Map;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ch.rakudave.jnetmap.net.PortScan;
import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.SwingHelper;
import ch.rakudave.jnetmap.util.logging.Logger;

@SuppressWarnings("serial")
public class PortScanner extends JDialog {
	private int port;

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public PortScanner(final Frame owner, String address) {
		super(owner, Lang.getNoHTML("port.scanner"), ModalityType.DOCUMENT_MODAL);
		final PortScanner _this = this;
		setLayout(new BorderLayout(5, 5));
		setPreferredSize(new Dimension(500,500));
		JPanel main = new JPanel(new BorderLayout(5, 5));
			JPanel listHolder = new JPanel(new BorderLayout()); // I hate JTables. With a passion.
				final JList leftList = new JList();
					leftList.setPreferredSize(new Dimension(55, 400));
				final JList rightList = new JList();
					leftList.addListSelectionListener(new ListSelectionListener() {
						@Override
						public void valueChanged(ListSelectionEvent e) {
							rightList.setSelectedIndex(leftList.getSelectedIndex());
							try {
								if (_this != null) _this.port = Integer.valueOf((String) leftList.getSelectedValue());
							} catch (Exception ex) {
								Logger.debug("Unable to get int-value from list", ex);
							}
						}
					});
					rightList.addListSelectionListener(new ListSelectionListener() {
						@Override
						public void valueChanged(ListSelectionEvent e) {
							leftList.setSelectedIndex(rightList.getSelectedIndex());
						}
					});
				listHolder.add(leftList, BorderLayout.WEST);
				listHolder.add(rightList, BorderLayout.CENTER);
			JPanel topPanel = new JPanel(new BorderLayout());
				JPanel topInner = new JPanel(new GridLayout(0, 1, 5, 5));
					final JTextField addr = new JTextField(address);
					final JComboBox scanType = new JComboBox(new String[] {Lang.get("port.wellknown"), Lang.get("port.registered"), Lang.get("port.all"), Lang.get("port.manual")});
					final JPanel manualPorts = new JPanel(new GridLayout(1, 0, 5, 5));
						final JSpinner fromPort = new JSpinner(new SpinnerNumberModel(0,0,65535,1));
							fromPort.setEnabled(false);
							fromPort.setEditor(new JSpinner.NumberEditor(fromPort, "#"));
						final JSpinner toPort = new JSpinner(new SpinnerNumberModel(1023,0,65535,1));
							toPort.setEnabled(false);
							toPort.setEditor(new JSpinner.NumberEditor(toPort, "#"));
						manualPorts.add(fromPort);
						manualPorts.add(toPort);
					scanType.addActionListener(new ActionListener() {
						@Override
						public void actionPerformed(ActionEvent e) {
							fromPort.setEnabled(scanType.getSelectedIndex() == 3);
							toPort.setEnabled(scanType.getSelectedIndex() == 3);
						}
					});
					topInner.add(addr);
					topInner.add(scanType);
					topInner.add(manualPorts);
				JButton scan = new JButton(new AbstractAction(Lang.get("port.scan")) {
					@Override
					public void actionPerformed(ActionEvent e) {
						InetAddress inetAddr;
						try {
							inetAddr = InetAddress.getByName(addr.getText());
						} catch (UnknownHostException e1) {
							JOptionPane.showMessageDialog(null, Lang.get("address.invalid"), Lang.get("message.error"), JOptionPane.ERROR_MESSAGE);
							return;
						}
						Map<Integer, String> map;
						if (scanType.getSelectedIndex() == 0) {
							map = PortScan.wellKnownPortsScan(inetAddr);
						} else if (scanType.getSelectedIndex() == 1) {
							map = PortScan.registeredPortscan(inetAddr);
						} else if (scanType.getSelectedIndex() == 1) {
							map = PortScan.allPortsScan(inetAddr);
						} else {
							map = PortScan.scan(inetAddr, Integer.valueOf(fromPort.getValue().toString()), Integer.valueOf(toPort.getValue().toString()));
						}
						Vector<String> leftData = new Vector<String>();
						Vector<String> rightData = new Vector<String>();
						for (int i : new TreeSet<Integer>(map.keySet())) {
							leftData.add(String.valueOf(i));
							rightData.add(map.get(i));
						}
						leftList.setListData(leftData);
						rightList.setListData(rightData);
					}
				});
				scan.setPreferredSize(new Dimension(70, 100));
				topPanel.add(topInner, BorderLayout.CENTER);
				topPanel.add(scan, BorderLayout.EAST);
			main.add(topPanel, BorderLayout.NORTH);
			main.add(listHolder);
		JPanel bottomRow = new JPanel(new FlowLayout(FlowLayout.TRAILING, 5, 5));
			JButton ok = new JButton(Lang.get("action.ok"), Icons.get("ok"));
				ok.setPreferredSize(new Dimension(60, 30));
				ok.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						_this.dispose();
					}
				});
			bottomRow.add(ok);
		add(main, BorderLayout.CENTER);
		add(bottomRow, BorderLayout.SOUTH);
		pack();
		SwingHelper.centerTo(owner, this);
		setVisible(true);
	}
	
	public int getPort() {
		return port;
	}
}
