package ch.rakudave.jnetmap.view.preferences;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileNameExtensionFilter;

import ch.rakudave.jnetmap.controller.Controller;
import ch.rakudave.jnetmap.net.status.Status;
import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.SwingHelper;
import ch.rakudave.jnetmap.util.logging.Logger;
import ch.rakudave.jnetmap.view.jung.EdgeTransformers;

/**
 * @author rakudave
 */
@SuppressWarnings("serial")
public class AppearancePanel extends PreferencePanel {
	private Window owner;
	@SuppressWarnings("rawtypes")
	private JComboBox iconSelector, deviceSelector, shapeSelector;
	private JPanel upButton, downButton, unknownButton, not_foundButton;
	private JCheckBox deviceLabelName, deviceLabelDescription, deviceLabelLocation, deviceLabelVendor, 
						deviceLabelModel, deviceLabelIP, deviceLabelMAC, connectionLabelLatency;
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public AppearancePanel(Window parent) {
		owner = parent; 
		title = Lang.getNoHTML("preferences.appearance");
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		// icons
		JPanel iconWrapper = new JPanel(new GridLayout(1, 2, 5, 5)) {{
			setMaximumSize(new Dimension(500,25));
			setBorder(BorderFactory.createTitledBorder(Lang.getNoHTML("preferences.general.iconstyle") + 
					" " + Lang.getNoHTML("message.restart")));
			iconSelector = new JComboBox(Icons.getThemes());
				iconSelector.setRenderer(new IconListRenderer());
				iconSelector.setSelectedItem(Icons.currentTheme());
			add(iconSelector);
			add(SwingHelper.createAlignedButton(new AbstractAction(Lang.get("action.install"), Icons.get("install")) {
				@Override
				public void actionPerformed(ActionEvent e) {
					File zip = SwingHelper.openDialog(owner, new FileNameExtensionFilter("ZIP-file", "zip"));
					if (zip != null) {
						if (Icons.importTheme(zip)) {
							iconSelector.addItem(zip.getName().replace(".zip", ""));
							Logger.info("Successfully imported icon-theme " + iconSelector.getSelectedItem());
						} else {
							Logger.error("Failed to import icon-theme " + zip.getName());
							JOptionPane.showMessageDialog(owner, Lang.get("message.import.error"), 
									zip.getName(), JOptionPane.ERROR_MESSAGE);
						}
					}
				}
			}));
		}};
		// device icons
		JPanel deviceIconWrapper = new JPanel(new GridLayout(1, 2, 5, 5)) {{
			setMaximumSize(new Dimension(500,25));
			setBorder(BorderFactory.createTitledBorder(Lang.getNoHTML("preferences.general.devicestyle")));
			deviceSelector = new JComboBox(Icons.getDeviceThemes());
				deviceSelector.setRenderer(new DeviceIconListRenderer());
				deviceSelector.setSelectedItem(Icons.currentDeviceTheme());
			add(deviceSelector);
			add(SwingHelper.createAlignedButton(new AbstractAction(Lang.get("action.install"), Icons.get("install")) {
				@Override
				public void actionPerformed(ActionEvent e) {
					File zip = SwingHelper.openDialog(owner, new FileNameExtensionFilter("ZIP-file", "zip"));
					if (zip != null) {
						if (Icons.importDeviceTheme(zip)) {
							deviceSelector.addItem(zip.getName().replace(".zip", ""));
							Logger.info("Successfully imported icon-theme " + deviceSelector.getSelectedItem());
						} else {
							Logger.error("Failed to import device-theme " + zip.getName());
							JOptionPane.showMessageDialog(owner, Lang.get("message.import.error"), 
									zip.getName(), JOptionPane.ERROR_MESSAGE);
						}
					}
				}
			}));
		}};
		// colors
		JPanel colors = new JPanel(new GridLayout(0, 3, 5, 5));
			colors.setBorder(BorderFactory.createTitledBorder(Lang.getNoHTML("preferences.appearance.color")));
			colors.setMaximumSize(new Dimension(2000, 120));
			upButton = new JPanel();
				upButton.setBackground(Status.UP.getColor());
				upButton.addMouseListener(colorSelector(upButton));
				upButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			downButton = new JPanel();
				downButton.setBackground(Status.DOWN.getColor());
				downButton.addMouseListener(colorSelector(downButton));
				downButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			unknownButton = new JPanel();
				unknownButton.setBackground(Status.UNKNOWN.getColor());
				unknownButton.addMouseListener(colorSelector(unknownButton));
				unknownButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			not_foundButton = new JPanel();
				not_foundButton.setBackground(Status.NOT_FOUND.getColor());
				not_foundButton.addMouseListener(colorSelector(not_foundButton));
				not_foundButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			colors.add(new JLabel(Status.UP.getMessage())); 
				colors.add(upButton); colors.add(createResetButton(upButton, Color.green));
			colors.add(new JLabel(Status.NOT_FOUND.getMessage()));
				colors.add(not_foundButton); colors.add(createResetButton(not_foundButton, Color.orange));
			colors.add(new JLabel(Status.DOWN.getMessage()));
				colors.add(downButton); colors.add(createResetButton(downButton, Color.red));
			colors.add(new JLabel(Status.UNKNOWN.getMessage()));
				colors.add(unknownButton); colors.add(createResetButton(unknownButton, Color.gray));
		// Connection
			JPanel shapeWrapper = new JPanel(new GridLayout(0, 2, 5, 5)) {{
				setBorder(BorderFactory.createTitledBorder(Lang.getNoHTML("connection")));
				shapeSelector = new JComboBox(EdgeTransformers.Shape.values());
					shapeSelector.setSelectedItem(EdgeTransformers.Shape.valueOf(Settings.get("edge.shape", "Quad")));
				connectionLabelLatency = new JCheckBox(Lang.get("connection.latency"), Settings.getBoolean("connection.label.latency", false));
				add(shapeSelector);
				add(connectionLabelLatency);
			}};
		// Device label
			JPanel deviceWrapper = new JPanel(new GridLayout(0, 3, 5, 5)) {{
				setBorder(BorderFactory.createTitledBorder(Lang.getNoHTML("preferences.appearance.device.label")));
				deviceLabelName = new JCheckBox(Lang.get("device.name"), Settings.getBoolean("device.label.name", true));
				deviceLabelDescription = new JCheckBox(Lang.get("device.description"), Settings.getBoolean("device.label.description", false));
				deviceLabelLocation = new JCheckBox(Lang.get("device.location"), Settings.getBoolean("device.label.location", false));
				deviceLabelVendor = new JCheckBox(Lang.get("device.vendor"), Settings.getBoolean("device.label.vendor", false));
				deviceLabelModel = new JCheckBox(Lang.get("device.model"), Settings.getBoolean("device.label.model", false));
				deviceLabelIP = new JCheckBox(Lang.get("interface.address"), Settings.getBoolean("device.label.ip", false));
				deviceLabelMAC = new JCheckBox(Lang.get("interface.mac"), Settings.getBoolean("device.label.mac", false));
				add(deviceLabelName);
				add(deviceLabelDescription);
				add(deviceLabelLocation);
				add(deviceLabelVendor);
				add(deviceLabelModel);
				add(deviceLabelIP);
				add(deviceLabelMAC);
			}};
		add(iconWrapper);
		add(Box.createVerticalStrut(5));
		add(deviceIconWrapper);
		add(Box.createVerticalStrut(5));
		add(colors);
		add(Box.createVerticalStrut(5));
		add(deviceWrapper);
		add(Box.createVerticalStrut(5));
		add(shapeWrapper);
		add(Box.createVerticalGlue());
	}

	@Override
	public void save() {
		Settings.put("icon.theme", iconSelector.getSelectedItem().toString());
		Settings.put("device.theme", deviceSelector.getSelectedItem().toString());
		Icons.loadDevices(deviceSelector.getSelectedItem().toString());
		Status.UP.setColor(upButton.getBackground());
		Status.DOWN.setColor(downButton.getBackground());
		Status.UNKNOWN.setColor(unknownButton.getBackground());
		Status.NOT_FOUND.setColor(not_foundButton.getBackground());
		Settings.put("device.label.name", deviceLabelName.isSelected());
		Settings.put("device.label.description", deviceLabelDescription.isSelected());
		Settings.put("device.label.location", deviceLabelLocation.isSelected());
		Settings.put("device.label.vendor", deviceLabelVendor.isSelected());
		Settings.put("device.label.model", deviceLabelModel.isSelected());
		Settings.put("device.label.ip", deviceLabelIP.isSelected());
		Settings.put("device.label.mac", deviceLabelMAC.isSelected());
		Settings.put("connection.label.latency", connectionLabelLatency.isSelected());
		Settings.put("edge.shape", shapeSelector.getSelectedItem().toString());
		Controller.refreshAll();
	}
	
	private MouseListener colorSelector(final JComponent button) {
		return new MouseListener() {
			@Override public void mouseReleased(MouseEvent e) {}
			@Override public void mousePressed(MouseEvent e) {}
			@Override public void mouseExited(MouseEvent e) {}
			@Override public void mouseEntered(MouseEvent e) {}
			@Override
			public void mouseClicked(MouseEvent e) {
				Color initialBackground = button.getBackground();
		        Color background = JColorChooser.showDialog(owner,
		            "JColorChooser", initialBackground);
		        if (background != null) {
		          button.setBackground(background);
		        }
			}
		};
	}

	@SuppressWarnings({ "rawtypes" })
	private class IconListRenderer extends DefaultListCellRenderer {
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			JLabel label = (JLabel) super.getListCellRendererComponent(list,
					value, index, isSelected, cellHasFocus);
			if (value == null) return label;
			try {
				String theme = (String) value; 
				label.setIcon(new ImageIcon(new File(Icons.iconDir, theme+"/jnetmap_small.png").getAbsolutePath()));
			} catch (Exception e) {
				Logger.trace("Unable to get icon", e);
			}
			return label;
		}
	}

	@SuppressWarnings({ "rawtypes" })
	private class DeviceIconListRenderer extends DefaultListCellRenderer {
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			JLabel label = (JLabel) super.getListCellRendererComponent(list,
					value, index, isSelected, cellHasFocus);
			if (value == null) return label;
			try {
				String theme = (String) value; 
				label.setIcon(new ImageIcon(new File(Icons.devicesDir, theme+"/workstation_small.png").getAbsolutePath()));
			} catch (Exception e) {
				Logger.trace("Unable to get icon", e);
			}
			return label;
		}
	}
	
	private JButton createResetButton(final JComponent display, final Color color) {
		return new JButton(new AbstractAction(Lang.get("action.reset")) {
			@Override
			public void actionPerformed(ActionEvent e) {
				display.setBackground(color);
			}
		});
	}
}
